==> open-xchange-documents-backend_7.10.0.orig.tar.bz2: open-xchange-documents-backend-7.10.0/com.openexchange.office.tools.common/src/com/openexchange/office/tools/common/string/UnicodeValidator.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.tools.common.string;

public class UnicodeValidator {

    /*
     * This method checks if there is invalid Unicode used (high surrogate without low surrogate or
     * low surrogate without high surrogate)). These invalid surrogates are replaced by replaceChar.
     * The fixed String is then returned, in case nothing was changed the output string is equal
     * to the input string.
     */
    public static String replaceInvalidCharacters(String s, char replaceChar) {
        if(s!=null) {
            for(int i=0; i<s.length(); i++) {
                final char c = s.charAt(i);
                if(Character.isHighSurrogate(c)) {
                    if(i+1==s.length()||!Character.isLowSurrogate(s.charAt(i+1))) {
                        final StringBuffer buffer = new StringBuffer(s);
                        buffer.setCharAt(i, replaceChar);
                        s = buffer.toString();
                    }
                }
                else if(Character.isLowSurrogate(c)) {
                    if(i==0||!Character.isHighSurrogate(s.charAt(i-1))) {
                        final  StringBuffer buffer = new StringBuffer(s);
                        buffer.setCharAt(i, replaceChar);
                        s = buffer.toString();
                    }
                }
            }
        }
        return s;
    }

    public static String replaceComposedUnicodeCharacters(String s, char replaceChar) {
        String result = s;

        if(s!=null) {
            final StringBuilder buffer = new StringBuilder(s);

            for(int i=0; i<s.length(); i++) {
                final char c = s.charAt(i);
                if(Character.isHighSurrogate(c) ||
                  (Character.isLowSurrogate(c))) {
                    buffer.setCharAt(i, replaceChar);
                }
            }

            result = buffer.toString();
        }
        return result;
    }
}

==> open-xchange-documents-backend_7.10.3.orig.tar.bz2: open-xchange-documents-backend-7.10.3/com.openexchange.office.tools.common/src/com/openexchange/office/tools/common/string/UnicodeValidator.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.office.tools.common.string;

public class UnicodeValidator {

    private UnicodeValidator() {}

    /*
     * This method checks if there is invalid Unicode used (high surrogate without low surrogate or
     * low surrogate without high surrogate)). These invalid surrogates are replaced by replaceChar.
     * The fixed String is then returned, in case nothing was changed the output string is equal
     * to the input string.
     */
    public static String replaceInvalidCharacters(String s, char replaceChar) {
        if(s!=null) {
            for(int i=0; i<s.length(); i++) {
                final char c = s.charAt(i);
                if(Character.isHighSurrogate(c)) {
                    if(i+1==s.length()||!Character.isLowSurrogate(s.charAt(i+1))) {
                        final StringBuilder buffer = new StringBuilder(s);
                        buffer.setCharAt(i, replaceChar);
                        s = buffer.toString();
                    }
                }
                else if(Character.isLowSurrogate(c)) {
                    if(i==0||!Character.isHighSurrogate(s.charAt(i-1))) {
                        final  StringBuilder buffer = new StringBuilder(s);
                        buffer.setCharAt(i, replaceChar);
                        s = buffer.toString();
                    }
                }
            }
        }

        return s;
    }

    public static String replaceComposedUnicodeCharacters(String s, char replaceChar) {
        String result = s;

        if(s!=null) {
            final StringBuilder buffer = new StringBuilder(s);

            for(int i=0; i<s.length(); i++) {
                final char c = s.charAt(i);
                if(Character.isHighSurrogate(c) ||
                  (Character.isLowSurrogate(c))) {
                    buffer.setCharAt(i, replaceChar);
                }
            }

            result = buffer.toString();
        }
        return result;
    }
}
